#include "algo.h"
#include <iostream>

const int N = 10000000;

int main(int argc, char *argv[]) {    
    omp_set_num_threads(2);
	__cilkrts_set_param("nworkers","2");
	
	std :: cout << "Array creation..." << std :: endl;
    SimpleArray data(N);
    data.Fill();

    Sum *algo;
    // 1. At first try serial algorithm
    std :: cout << "Serial reduction:" << std :: endl;
    algo = new Sum;
    algo->RunAndMeasure(data);
    delete algo;
    // 2. Now do in parallel using OpenMP
	/*
    std :: cout << "OpenMP reduction:" << std :: endl;
    algo = new OpenMPSum();
    algo->RunAndMeasure(data);
    delete algo;    
    */
	// 3. Now do in parallel using cilk_for keyword (using reducer)
    /*
	std :: cout << "cilk_for reduction:" << std :: endl;
    algo = new CilkForSum();
    algo->RunAndMeasure(data);
    delete algo;
    */
	// 4. Now do in parallel using cilk_for keyword (NOT using reducers)
    /*
	std :: cout << "cilk_for reduction (no reducers, erronous result):" << std :: endl;
    algo = new CilkForNoReducerSum();
    algo->RunAndMeasure(data);
    delete algo;            
    */
	// 5. Now do in parallel using cilk_spawn 
    /*
	std :: cout << "cilk_spawn reduction (using " << 2 << " threads)" << ":" << std :: endl;
    algo = new CilkSpawnSum(4);
    algo->RunAndMeasure(data);
    delete algo;
	*/

    return 0;
}
